@@ -1,7 +1,7 @@
 #
 # This file is part of Checkbox.
 #
-# Copyright 2011-2013 Canonical Ltd.
+# Copyright 2011-2016 Canonical Ltd.
 #
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
@@ -22,6 +22,7 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 from collections import OrderedDict
+from subprocess import check_output, CalledProcessError
 import re
 import string
 
@@ -87,30 +88,70 @@ FLASH_RE = re.compile(r"Flash", re.I)
 FLASH_DISK_RE = re.compile(r"Mass|Storage|Disk", re.I)
 
 
+def slugify(_string):
+    """Transform any string to one that can be used in job IDs."""
+    valid_chars = frozenset(
+        "-_.{}{}".format(string.ascii_letters, string.digits))
+    return ''.join(c if c in valid_chars else '_' for c in _string)
+
+
+def find_pkname_is_root_mountpoint(devname, lsblk=None):
+    """Check for partition mounted as root for a DISK device."""
+    if lsblk:
+        try:
+            lsblk = lsblk.read()
+        except AttributeError:
+            pass
+        for line in lsblk.splitlines():
+            if (
+                line.endswith('MOUNTPOINT="/"') and
+                line.startswith('KNAME="{}'.format(devname))
+            ):
+                return True
+            if (
+                line.endswith('MOUNTPOINT="/writable"') and
+                line.startswith('KNAME="{}'.format(devname))
+            ):
+                return True
+    return False
+
+
 class UdevadmDevice(object):
     __slots__ = (
         "_environment",
         "_name",
+        "_lsblk",
         "_bits",
         "_stack",
         "_bus",
         "_interface",
+        "_mac",
         "_product",
         "_product_id",
+        "_subproduct_id",
+        "_product_slug",
         "_vendor",
-        "_vendor_id",)
+        "_vendor_id",
+        "_subvendor_id",
+        "_vendor_slug",)
 
-    def __init__(self, environment, name, bits=None, stack=[]):
+    def __init__(self, environment, name, lsblk=None, bits=None, stack=[]):
         self._environment = environment
         self._name = name
+        self._lsblk = lsblk
         self._bits = bits
         self._stack = stack
         self._bus = None
         self._interface = None
+        self._mac = None
         self._product = None
         self._product_id = None
+        self._subproduct_id = None
+        self._product_slug = None
         self._vendor = None
         self._vendor_id = None
+        self._subvendor_id = None
+        self._vendor_slug = None
 
     def __repr__(self):
         vid = int(self.vendor_id) if self.vendor_id else 0
@@ -144,6 +185,13 @@ class UdevadmDevice(object):
         if bus == 'input' and any(d.bus == 'usb' for d in self._stack):
             bus = 'usb'
 
+        # Only keep one device per udev path
+        if (
+            bus == 'bluetooth' and
+            [d for d in reversed(self._stack) if d.category == "BLUETOOTH"]
+        ):
+            return None
+
         return bus
 
     @bus.setter
@@ -157,6 +205,8 @@ class UdevadmDevice(object):
                 devtype = self._environment["DEVTYPE"]
                 if devtype in ("wlan", "wimax"):
                     return "WIRELESS"
+                elif devtype == "wwan":
+                    return "WWAN"
             # Ralink and realtek SDIO wireless
             if "INTERFACE" in self._environment:
                 if (self.driver and
@@ -301,8 +351,21 @@ class UdevadmDevice(object):
         if self.driver:
             if self.driver.startswith("sdhci"):
                 return "CARDREADER"
+            # Only consider eMMC internal drives as DISK.
+            # As it's not possible to distinguish them from simple MMC
+            # removable storage, only those with a partition mounted as /
+            # will be considered.
+            # To avoid categorizing SD cards as main storage and run heavy
+            # tests on them (e.g on a pandaboard) only devices with the udev
+            # property MMC_TYPE == MMC are accepted.
             if self.driver.startswith("mmc"):
-                return "CARDREADER"
+                if (
+                    self._mmc_type == 'MMC' and
+                    find_pkname_is_root_mountpoint(self.name, self._lsblk)
+                ):
+                    return "DISK"
+                else:
+                    return "CARDREADER"
             if self.driver == "rts_pstor":
                 return "CARDREADER"
             # See http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=702145
@@ -325,14 +388,18 @@ class UdevadmDevice(object):
                 # will be given the category attribute 'DISK' by udev_resource
                 if any(FLASH_RE.search(k) for k in self._environment.keys()):
                     return "CARDREADER"
-                if any(d.bus == 'usb' for d in self._stack):
-                    if (self.product is not None and
-                            CARD_READER_RE.search(self.product)):
-                        return "CARDREADER"
-                    if (self.vendor is not None and
-                            GENERIC_RE.search(self.vendor) and
-                            not FLASH_DISK_RE.search(self.product)):
-                        return "CARDREADER"
+            if any(d.bus == 'usb' for d in self._stack):
+                if (self.product is not None and
+                        CARD_READER_RE.search(self.product)):
+                    return "CARDREADER"
+                if (self.vendor is not None and
+                        GENERIC_RE.search(self.vendor) and
+                        not FLASH_DISK_RE.search(self.product)):
+                    return "CARDREADER"
+            # A rare gem, this driver reported by udev is actually an ID_MODEL:
+            # E: DRIVER=Realtek PCIe card reader
+            if re.search(r'card.*reader', self.driver, re.I):
+                return "CARDREADER"
 
         if "ID_TYPE" in self._environment:
             id_type = self._environment["ID_TYPE"]
@@ -341,6 +408,7 @@ class UdevadmDevice(object):
             if (
                 id_type == "disk"
                 and not any(d.category == "CARDREADER" for d in self._stack)
+                and not any(d.bus == 'usb' for d in self._stack)
             ):
                 return "DISK"
             if not any(d.bus == 'usb' for d in self._stack):
@@ -358,12 +426,21 @@ class UdevadmDevice(object):
                     # A QEMU/KVM virtual disk, but should be treated
                     # as DISK nonetheless
                     return "DISK"
-                if self.driver == 'nvme' and self.bus == 'pci':
+                if self.driver == 'nvme' and self.bus in ('pci', 'nvme'):
                     # NVMe device in PCIe bus, this should also be
                     # treated as DISK as it presents block devices
                     # we need to test, and is a valid disk device
                     # we need to report.
                     return "DISK"
+                if '/dev/mapper' in self._environment.get('DEVLINKS', ''):
+                    if "ID_FS_TYPE" in self._environment:
+                        if self._environment["ID_FS_TYPE"] != 'swap':
+                            return "DISK"
+                    else:
+                        return "DISK"
+                if self.driver == 'dasd-eckd':
+                    # IBM s390x DASD device types
+                    return "DISK"
             if devtype == "scsi_device":
                 match = SCSI_RE.match(self._environment.get("MODALIAS", ""))
                 type = int(match.group("type"), 16) if match else -1
@@ -389,6 +466,10 @@ class UdevadmDevice(object):
             if self._environment["DRIVER"] == "floppy":
                 return "FLOPPY"
 
+        if "DEVLINKS" in self._environment:
+            if "canbus" in self._environment["DEVLINKS"]:
+                return "CANBUS"
+
         # Some audio and serial devices have a product but no vendor
         # or product id. Special-case their categories for backwards-
         # compatibility.
@@ -412,15 +493,26 @@ class UdevadmDevice(object):
         # those devices.
         return None
 
+    @property
+    def major(self):
+        # See http://pad.lv/1559189
+        # We need the Major to identify IBM s390 disk devices (DASDs)
+        if "MAJOR" in self._environment:
+            return self._environment["MAJOR"]
+
     @property
     def driver(self):
         if "DRIVER" in self._environment:
             return self._environment["DRIVER"]
-        # Check parent device for driver
+        # Check parent devices for driver
         if self._stack:
             parent = self._stack[-1]
             if "DRIVER" in parent._environment:
                 return parent._environment["DRIVER"]
+            if self.bus == 'nvme' and parent._stack:
+                parent = parent._stack[-1]
+                if "DRIVER" in parent._environment:
+                    return parent._environment["DRIVER"]
         return None
 
     @property
@@ -440,6 +532,22 @@ class UdevadmDevice(object):
         """
         return self._environment.get("DEVPATH")
 
+    @property
+    def _mmc_type(self):
+        """
+        Return the MMC type available in the stack.
+
+        This is used internally by UdevadmParser only
+        """
+        if "MMC_TYPE" in self._environment:
+            return self._environment["MMC_TYPE"]
+        # Check parent device for driver
+        if self._stack:
+            parent = self._stack[-1]
+            if "MMC_TYPE" in parent._environment:
+                return parent._environment["MMC_TYPE"]
+        return None
+
     @property
     def product_id(self):
         if self._product_id is not None:
@@ -463,6 +571,18 @@ class UdevadmDevice(object):
         match = INPUT_RE.match(self._environment.get("MODALIAS", ""))
         if match:
             return int(match.group("product_id"), 16)
+        # disk
+        if self.driver == "nvme" and self.bus in ('pci', 'misc') and self._stack:
+            parent = self._stack[-1]
+            return parent.product_id
+        elif self.driver == "nvme" and self.bus == 'nvme' and self._stack:
+            parent = self._stack[-2]
+            return parent.product_id
+        # canbus
+        if "DEVLINKS" in self._environment:
+            if "canbus" in self._environment["DEVLINKS"]:
+                if "ID_MODEL_ID" in self._environment:
+                    return decode_id(self._environment["ID_MODEL_ID"])
         return None
 
     @product_id.setter
@@ -491,6 +611,18 @@ class UdevadmDevice(object):
             if vendor_id and vendor_id < 9:
                 vendor_id = 9
             return vendor_id
+        # disk
+        if self.driver == "nvme" and self.bus in ('pci', 'misc') and self._stack:
+            parent = self._stack[-1]
+            return parent.vendor_id
+        elif self.driver == "nvme" and self.bus == 'nvme' and self._stack:
+            parent = self._stack[-2]
+            return parent.vendor_id
+        # canbus
+        if "DEVLINKS" in self._environment:
+            if "canbus" in self._environment["DEVLINKS"]:
+                if "ID_VENDOR_ID" in self._environment:
+                    return decode_id(self._environment["ID_VENDOR_ID"])
         return None
 
     @vendor_id.setter
@@ -499,18 +631,62 @@ class UdevadmDevice(object):
 
     @property
     def subproduct_id(self):
+        if self._subproduct_id is not None:
+            return self._subproduct_id
         if "PCI_SUBSYS_ID" in self._environment:
             pci_subsys_id = self._environment["PCI_SUBSYS_ID"]
             subvendor_id, subproduct_id = pci_subsys_id.split(":")
             return int(subproduct_id, 16)
+        if self.driver == "nvme" and self.bus in ('pci', 'misc') and self._stack:
+            parent = self._stack[-1]
+            return parent.subproduct_id
+        elif self.driver == "nvme" and self.bus == 'nvme' and self._stack:
+            parent = self._stack[-2]
+            return parent.subproduct_id
         return None
 
+    @subproduct_id.setter
+    def subproduct_id(self, value):
+        self._subproduct_id = value
+
     @property
     def subvendor_id(self):
+        if self._subvendor_id is not None:
+            return self._subvendor_id
         if "PCI_SUBSYS_ID" in self._environment:
             pci_subsys_id = self._environment["PCI_SUBSYS_ID"]
             subvendor_id, subproduct_id = pci_subsys_id.split(":")
             return int(subvendor_id, 16)
+        if self.driver == "nvme" and self.bus in ('pci', 'misc') and self._stack:
+            parent = self._stack[-1]
+            return parent.subvendor_id
+        elif self.driver == "nvme" and self.bus == 'nvme' and self._stack:
+            parent = self._stack[-2]
+            return parent.subvendor_id
+        return None
+
+    @subvendor_id.setter
+    def subvendor_id(self, value):
+        self._subvendor_id = value
+
+    @property
+    def product_slug(self):
+        """Returns a version of the product name trimmed from any weird characters."""
+        if self._product_slug is not None:
+            return self._product_slug
+        if self.product is not None:
+            return slugify(self.product)
+
+        return None
+
+    @property
+    def vendor_slug(self):
+        """Returns a version of the vendor name trimmed from any weird characters."""
+        if self._vendor_slug is not None:
+            return self._vendor_slug
+        if self.vendor is not None:
+            return slugify(self.vendor)
+
         return None
 
     @property
@@ -525,6 +701,21 @@ class UdevadmDevice(object):
         elif (self._environment.get("DEVTYPE") == "disk" and
                 "ID_MODEL_ENC" in self._environment):
             return decode_id(self._environment["ID_MODEL_ENC"])
+        elif self.driver == "nvme" and self.bus in ('pci', 'misc'):
+            return self.name
+        elif self.driver == "nvme" and self.bus == 'nvme' and self._stack:
+            parent = self._stack[-2]
+            if parent.product:
+                return parent.product
+            else:
+                return self.name
+        elif (
+            self._environment.get("DEVTYPE") == "disk" and
+            self.driver == 'virtio_blk' and self.bus == 'virtio'):
+            return self.name
+        elif self.major == "94":
+            # See http://pad.lv/1559189
+            return "IBM s390 Virtual Disk"
 
         # floppy
         if self.driver == "floppy":
@@ -567,10 +758,24 @@ class UdevadmDevice(object):
             if "ID_MODEL_ENC" in self._environment:
                 return decode_id(self._environment["ID_MODEL_ENC"])
 
+        if self.driver == 'mmcblk':
+            # Check parent device for MMC name
+            if self._stack:
+                parent = self._stack[-1]
+                if "MMC_NAME" in parent._environment:
+                    return parent._environment["MMC_NAME"]
+
         for element in ("NAME", "POWER_SUPPLY_MODEL_NAME"):
             if element in self._environment:
                 return self._environment[element].strip('"')
 
+        if "DEVLINKS" in self._environment:
+            if "canbus" in self._environment["DEVLINKS"]:
+                if "ID_MODEL_ENC" in self._environment:
+                    return decode_id(self._environment["ID_MODEL_ENC"])
+            if "/dev/mapper" in self._environment["DEVLINKS"]:
+                return self.name
+
         return None
 
     @product.setter
@@ -603,6 +808,12 @@ class UdevadmDevice(object):
 
         if "ID_VENDOR_FROM_DATABASE" in self._environment:
             return self._environment["ID_VENDOR_FROM_DATABASE"]
+        if self.driver == "nvme" and self.bus in ('pci', 'misc') and self._stack:
+            parent = self._stack[-1]
+            return parent.vendor
+        elif self.driver == "nvme" and self.bus == 'nvme' and self._stack:
+            parent = self._stack[-2]
+            return parent.vendor
 
         # bluetooth (if USB base class is vendor specific)
         if self.bus == 'bluetooth':
@@ -625,6 +836,11 @@ class UdevadmDevice(object):
             if "ID_VENDOR_ENC" in self._environment:
                 return decode_id(self._environment["ID_VENDOR_ENC"])
 
+        if "DEVLINKS" in self._environment:
+            if "canbus" in self._environment["DEVLINKS"]:
+                if "ID_VENDOR_ENC" in self._environment:
+                    return decode_id(self._environment["ID_VENDOR_ENC"])
+
         return None
 
     @vendor.setter
@@ -635,11 +851,29 @@ class UdevadmDevice(object):
     def interface(self):
         if self._interface is not None:
             return self._interface
-        if (self.category in ("NETWORK", "WIRELESS") and
-                "INTERFACE" in self._environment):
-            return self._environment["INTERFACE"]
+        if self.category in ("NETWORK", "WIRELESS", "WWAN"):
+            if "INTERFACE" in self._environment:
+                return self._environment["INTERFACE"]
+            else:
+                return 'UNKNOWN'
         return None
 
+    @property
+    def mac(self):
+        if self._mac is not None:
+            return self._mac
+        if self.category in ("NETWORK", "WIRELESS", "WWAN"):
+            if "ID_NET_NAME_MAC" in self._environment:
+                mac = self._environment["ID_NET_NAME_MAC"][3:]
+                return ':'.join([mac[i:i+2] for i in range(0, len(mac), 2)])
+            else:
+                return 'UNKNOWN'
+        return None
+
+    @mac.setter
+    def mac(self, value):
+        self._mac = value
+
     @interface.setter
     def interface(self, value):
         self._interface = value
@@ -647,7 +881,8 @@ class UdevadmDevice(object):
     def as_json(self):
         attributes = ("path", "bus", "category", "driver", "product_id",
                       "vendor_id", "subproduct_id", "subvendor_id", "product",
-                      "vendor", "interface", "name")
+                      "vendor", "interface", "mac", "name", "product_slug",
+                      "vendor_slug")
 
         return {a: getattr(self, a) for a in attributes if getattr(self, a)}
 
@@ -657,12 +892,28 @@ class UdevadmParser(object):
 
     device_factory = UdevadmDevice
 
-    def __init__(self, stream_or_string, bits=None):
+    def __init__(self, stream_or_string, lsblk=None, bits=None):
         self.stream_or_string = stream_or_string
+        self.lsblk = lsblk
         self.bits = bits
         self.devices = OrderedDict()
 
     def _ignoreDevice(self, device):
+        # See http://pad.lv/1559189
+        # s390 LPARs and zVM DASDs provide very little info via udev so we
+        # need to handle them before anything else, otherwise they'll never
+        # appear. These devices all have Major number 94, so this is the
+        # easiest way to locate them.
+        if device.major == "94":
+            return False
+
+        # Keep /dev/mapper devices (non swap)
+        if '/dev/mapper' in device._environment.get('DEVLINKS', ''):
+            if "ID_FS_TYPE" in device._environment:
+                if device._environment["ID_FS_TYPE"] == 'swap':
+                    return True
+            return False
+
         # Ignore devices without bus information
         if not device.bus:
             return True
@@ -675,14 +926,22 @@ class UdevadmParser(object):
         # Do not ignore nvme devices on the pci bus, these are to be treated
         # as disks (categorization is done elsewhere). Note that the *parent*
         # device will have no category, though it's not ignored per se.
-        if device.bus == 'pci' and device.driver == 'nvme':
+        if device.bus in ('pci', 'nvme') and device.driver == 'nvme':
             return False
-
-        # Do not ignore QEMU/KVM virtio disks
+        # Do not ignore eMMC drives (pad.lv/1522768)
         if ("ID_PART_TABLE_TYPE" in device._environment and
+           device.driver == 'mmcblk' and
+           device._mmc_type == 'MMC'):
+            return False
+        # Do not ignore QEMU/KVM virtio disks
+        if ("DEVTYPE" in device._environment and
            device.bus == "virtio" and
            device.driver == "virtio_blk"):
             return False
+        # Do not ignore Bluetooth devices w/o product & vendor ID.
+        # These can be virtual devices, yet still constitute valid devices.
+        if device.category == "BLUETOOTH":
+            return False
 
         # Ignore devices without product AND vendor information
         if (device.product is None and device.product_id is None and
@@ -697,10 +956,29 @@ class UdevadmParser(object):
                 and device.subvendor_id is None)):
             return True
 
+        # Ignore FLOPPY, DISK and CDROM devices without a DEVNAME
+        # (See pad.lv/1539041)
+        if (
+            (device.category in ('CDROM', 'DISK', 'FLOPPY')) and
+            "DEVNAME" not in device._environment):
+            return True
+
         # Ignore ACPI devices
         if device.bus == "acpi":
             return True
 
+        # Ignore virtual devices created by Dell iDRAC manager
+        # See pad.lv/1308702
+        if device.vendor == "iDRAC":
+            return True
+
+        # Ignore virtual devices created by Cisco CIMC manager
+        # See pad.lv/1585802
+        if (device.product == "Virtual FDD/HDD" or
+            device.product == "Virtual Floppy" or
+            device.product == "Virtual CD/DVD"):
+            return True
+
         return False
 
     def getAttributes(self, path):
@@ -761,7 +1039,7 @@ class UdevadmParser(object):
             environment.setdefault("DEVPATH", path)
 
             device = self.device_factory(
-                environment, name, self.bits, list(stack))
+                environment, name, self.lsblk, self.bits, list(stack))
             if not self._ignoreDevice(device):
                 if device._raw_path in self.devices:
                     if self.devices[device._raw_path].category == 'CARDREADER':
@@ -803,19 +1081,38 @@ class UdevadmParser(object):
                     self.devices[device._raw_path] = device
             stack.append(device)
 
+        dev_mapper_devices = []
+        for d in self.devices.values():
+            if d.category == 'DISK':
+                if '/dev/mapper' in d._environment.get('DEVLINKS', ''):
+                    if "ID_FS_TYPE" in d._environment:
+                        if d._environment["ID_FS_TYPE"] != 'swap':
+                            dev_mapper_devices.append(d)
+                    else:
+                        dev_mapper_devices.append(d)
+
         for device in list(self.devices.values()):
-            if device.category in ("NETWORK", "WIRELESS", "OTHER"):
+            if device.category in ("NETWORK", "WIRELESS", "WWAN", "OTHER"):
                 dev_interface = [
                     d for d in self.devices.values()
-                    if d.category in ("NETWORK", "WIRELESS") and
+                    if d.category in ("NETWORK", "WIRELESS", "WWAN") and
                     device._raw_path != d._raw_path and
-                    device._raw_path in d._raw_path
+                    device._raw_path + '/' in d._raw_path
                 ]
                 if dev_interface:
                     dev_interface = dev_interface.pop()
+                    if dev_interface.interface == 'UNKNOWN':
+                        continue
                     dev_interface.bus = device.bus
                     dev_interface.product_id = device.product_id
                     dev_interface.vendor_id = device.vendor_id
+                    dev_interface.subproduct_id = device.subproduct_id
+                    dev_interface.subvendor_id = device.subvendor_id
+                    self.devices.pop(device._raw_path, None)
+            # If dev/mapper list devices then they take precedence over the
+            # other block devices
+            if dev_mapper_devices and device.category == 'DISK':
+                if device not in dev_mapper_devices:
                     self.devices.pop(device._raw_path, None)
 
         [result.addDevice(device) for device in self.devices.values()]
@@ -850,20 +1147,27 @@ def known_to_be_video_device(vendor_id, product_id, pci_class, pci_subclass):
 
 class UdevResult(object):
     def __init__(self):
-        self.devices = {"device_list": []}
+        self.devices = []
 
     def addDevice(self, device):
-        self.devices["device_list"].append(device)
+        self.devices.append(device)
 
 
-def parse_udevadm_output(output, bits=None):
+def parse_udevadm_output(output, lsblk=None, bits=None):
     """
     Parse output of `LANG=C udevadm info --export-db`
 
     :returns: :class:`UdevadmParser` object that corresponds to the
     parsed input
     """
-    udev = UdevadmParser(output, bits)
+    if lsblk is None:
+        try:
+            lsblk = check_output(
+                ['lsblk', '-i', '-n', '-P', '-o', 'KNAME,TYPE,MOUNTPOINT'],
+                universal_newlines=True)
+        except CalledProcessError as exc:
+            lsblk = ''
+    udev = UdevadmParser(output, lsblk, bits)
     result = UdevResult()
     udev.run(result)
     return result.devices
